/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.0 of the PHP license,       |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_0.txt.                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author: Johannes Schlueter                                           |
  +----------------------------------------------------------------------+
*/

/* $Id: idn.c,v 1.1 2004/04/26 15:41:17 johannes Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_idn.h"

#include <stringprep.h>
#include <idna.h>

/* If you declare any globals in php_idn.h uncomment this:
ZEND_DECLARE_MODULE_GLOBALS(idn)
*/

/* True global resources - no need for thread safety here */
static int le_idn;

/* {{{ idn_functions[] */
function_entry idn_functions[] = {
	PHP_FE(idn_to_ascii,	NULL)
	PHP_FE(idn_to_utf8, NULL)
	{NULL, NULL, NULL}
};
/* }}} */

/* {{{ idn_module_entry
 */
zend_module_entry idn_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"idn",
	idn_functions,
	PHP_MINIT(idn),
	PHP_MSHUTDOWN(idn),
	PHP_RINIT(idn),		/* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(idn),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(idn),
#if ZEND_MODULE_API_NO >= 20010901
	"0.1", /* Replace with version number for your extension */
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_IDN
ZEND_GET_MODULE(idn)
#endif

/* {{{ PHP_INI
 */
/* Remove comments and fill if you need to have entries in php.ini
PHP_INI_BEGIN()
    STD_PHP_INI_ENTRY("idn.global_value",      "42", PHP_INI_ALL, OnUpdateLong, global_value, zend_idn_globals, idn_globals)
    STD_PHP_INI_ENTRY("idn.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_idn_globals, idn_globals)
PHP_INI_END()
*/
/* }}} */

/* {{{ php_idn_init_globals
 */
/* Uncomment this function if you have INI entries
static void php_idn_init_globals(zend_idn_globals *idn_globals)
{
	idn_globals->global_value = 0;
	idn_globals->global_string = NULL;
}
*/
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(idn)
{
	/* Check that the the version of the GNU libidn library */
	if (!stringprep_check_version(STRINGPREP_VERSION)) {
		return FAILURE;
	}
	
	REGISTER_LONG_CONSTANT("IDNA_STRINGPREP_ERROR",       IDNA_STRINGPREP_ERROR,       CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_PUNYCODE_ERROR",         IDNA_PUNYCODE_ERROR,         CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_CONTAINS_NON_LDH",       IDNA_CONTAINS_NON_LDH,       CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_CONTAINS_MINUS",         IDNA_CONTAINS_MINUS,         CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_INVALID_LENGTH",         IDNA_INVALID_LENGTH,         CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_NO_ACE_PREFIX",          IDNA_NO_ACE_PREFIX,          CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_ROUNDTRIP_VERIFY_ERROR", IDNA_ROUNDTRIP_VERIFY_ERROR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_CONTAINS_ACE_PREFIX",    IDNA_CONTAINS_ACE_PREFIX,    CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_ICONV_ERROR",            IDNA_ICONV_ERROR,            CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("IDNA_MALLOC_ERROR",           IDNA_MALLOC_ERROR,           CONST_CS | CONST_PERSISTENT);

	/* If you have INI entries, uncomment these lines 
	ZEND_INIT_MODULE_GLOBALS(idn, php_idn_init_globals, NULL);
	REGISTER_INI_ENTRIES();
	*/

	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(idn)
{
	/* uncomment this line if you have INI entries
	UNREGISTER_INI_ENTRIES();
	*/
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request start */
/* {{{ PHP_RINIT_FUNCTION
 */
PHP_RINIT_FUNCTION(idn)
{
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request end */
/* {{{ PHP_RSHUTDOWN_FUNCTION
 */
PHP_RSHUTDOWN_FUNCTION(idn)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(idn)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "GNU libidn support", "enabled");
	php_info_print_table_row(2, "GNU libidn version", STRINGPREP_VERSION);
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	DISPLAY_INI_ENTRIES();
	*/
}
/* }}} */

/* {{{ proto string idn_to_ascii(string arg)
   Converts an UTF-8  encoded string to corresponding ASCII name */
PHP_FUNCTION(idn_to_ascii)
{
	/* The domain name the user enters */
	char *domain = NULL;
	int domain_len, len;

	/* PHP reference for error code */
	zval *errorcode = NULL;

	/* For returning error messages */
	char string[256];

	/* libidn return values */
	char *p;
	int rc;
	size_t i;


	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|z", &domain, &domain_len, &errorcode) == FAILURE) {
		return;
	}

	if (errorcode != NULL && !PZVAL_IS_REF(errorcode)) {
		zend_error(E_WARNING, "Second parameter wasn't passed by reference");
		RETURN_FALSE;
	}

	rc = idna_to_ascii_8z(domain, &p, 0);
	if (rc != IDNA_SUCCESS) {
		free(p);

		len = sprintf(string, "Couldn't convert domainname");
		if (errorcode != NULL) {
			ZVAL_LONG(errorcode, rc);
		}

		zend_error(E_WARNING, string);
		RETURN_FALSE;
	}

	RETVAL_STRINGL(p, strlen(p), 1);
	free(p);
	return;
}
/* }}} */

/* {{{ proto string idn_to_utf8(string arg)
   Converts an ASCII compatible domain name to UTF-8 */
PHP_FUNCTION(idn_to_utf8)
{
	/* The domain name the user enters */
	char *domain = NULL;
	int domain_len, len;

	/* PHP reference for error code */
	zval *errorcode = NULL;

	/* For returning error messages */
	char string[256];

	/* libidn return values */
	char *p;
	int rc;
	size_t i;


	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|z", &domain, &domain_len, &errorcode) == FAILURE) {
		return;
	}

	if (errorcode != NULL && !PZVAL_IS_REF(errorcode)) {
		zend_error(E_WARNING, "Second parameter wasn't passed by reference");
		RETURN_FALSE;
	}

	rc = idna_to_unicode_8z8z(domain, &p, 0);
	if (rc != IDNA_SUCCESS) {
		free(p);

		len = sprintf(string, "Couldn't convert domainname");
		if (errorcode != NULL) {
			ZVAL_LONG(errorcode, rc);
		}

		zend_error(E_WARNING, string);
		RETURN_FALSE;
	}

	RETVAL_STRINGL(p, strlen(p), 1);
	free(p);
	return;
}
/* }}} */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
